#!/usr/bin/perl -w

# nagios plugin using openweathermap's JSON api
#
# (c) 2013 by Frederic Krueger / fkrueger-dev-checkopenweathermap@holics.at
#
# Licensed under Creative Commons Attribution-ShareAlike 3.0 Unported
# There is no warranty of any kind, explicit or implied, for anything this software does or does not do.
# 
# Updates for this piece of software could be available under the following URL:
#   http://dev.techno.holics.at/check_openweathermap/


## uses
use Data::Dumper;             # for debugging
use JSON qw( decode_json );
use LWP::Simple;

use lib "/usr/lib/nagios/plugins";
use lib "/usr/lib64/nagios/plugins";
use lib "/srv/nagios/libexec";
use utils qw (%ERRORS);


## globals
my $dflt_where = "London,uk";    # default

# sorry OWM, the api.openweathermap.org/data/2.5/ API did not return any data for most station_ids;
# also, the data structure on non-api openweathermap.org is much easier to implement (and has less infos) ;-)

my $locurl = "http://api.openweathermap.org/data/2.5/find?q=<LOCATION>&units=metric&mode=json";
my $idurl = "http://api.openweathermap.org/data/2.1/weather/station/<ID>?type=json";

my ($PROG_NAME, $PROG_VERSION) = ("check_openweathermap", "0.0.2");

my %opttranslation = (
  'loc' => {      # probably can be used as a general API 2.5 translation table
    '_BASE_' => "{'list'}[0]",
    'clouds' => 'clouds-all',
    'country' => 'sys-country',
    'humidity' => 'main-humidity',
    'humidity-current' => 'main-humidity',
    'id' => 'id',
    'latitude' => 'coord-lat',
    'longitude' => 'coord-lon',
    'name' => 'name',
    'pressure' => 'main-pressure',
    'pressure-current' => 'main-pressure',
    'temp' => 'main-temp',
    'temp-current' => 'main-temp',
    'temp-minimum' => 'main-temp_min',
    'temp-maximum' => 'main-temp_max',
    'unixdate' => 'dt',
    'weather' => 'weather-0-main',
    'weather-main' => 'weather-0-main',
    'weather-description' => 'weather-0-description',
    'weather-icon' => 'weather-0-icon',
    'weather-id' => 'weather-0-id',
    'wind' => 'wind-speed',
    'wind-current' => 'wind-speed',
    'wind-deg' => 'wind-deg',
    'wind-gust' => 'wind-gust',
    'wind-speed' => 'wind-speed',
  },
  'id' => {               # probably can be used as a general API 2.1 translation table
    '_BASE_' => '',
    'clouds' => 'clouds-all',
    'date' => 'date',
    'dewpoint' => 'calc-dewpoint',
    'dt' => 'dt',
    'humidex' => 'calc-humidex',
    'humidity' => 'main-humidity',
    'humidity-current' => 'main-humidity',
    'id' => 'id',
    'latitude' => 'coord-lat',
    'longitude' => 'coord-lon',
    'name' => 'name',
    'pressure' => 'main-pressure',
    'pressure-current' => 'main-pressure',
    'temp' => 'main-temp',
    'temp-current' => 'main-temp',
    'unixdate' => 'dt',
    'weather' => 'weather-0-name',
    'wind' => 'wind-speed',
    'wind-current' => 'wind-speed',
    'wind-deg' => 'wind-deg',
    'wind-gust' => 'wind-gust',
    'wind-speed' => 'wind-speed',
  }
);

## subs
sub usage
{
  my $msg = shift;
  print "\n";
  print "usage: $0 <locationstring> <wantedinfo>\n";
  print "\n";
  print "This plugin allow you to monitor data from weather stations partaking in the openweathermap project ( http://openweathermap.org/ ).\n";
  print "\n";
  print "    locationstring is usually 'townname,countrytld' (ie. Wuerzburg,de or London,uk),\n";
  print "    but can be id=<owm-id> as well.\n";
  print "\n";
  print "    wantedinfo is either 'all' or one of the following:\n";
  print "      sys-country,sys-sunrise,sys-sunset (in epoch time)\n";
  print "      weather-main (ie. 'Clouds'), weather-description (ie. 'Broken clouds'), clouds-all (ie. '68' percent)\n";
  print "      main-temp,main-humidity,main-pressure,main-temp_min,main-temp_max,wind-speed,wind-gust,wind-deg (just the stated)\n";
  print "      name (name of location), coord-lat (latitude of location), coord-lon (longitude of location), id (id of weatherstation)\n";
  print "\n";
  print "    perfdata is being created with all available data, all the time.\n";
  print "\n";
  print "\n";
  ## calling example
#  print "Example with nearly all parameters available (as of 2013-07-28) being used:\n";
#  print "nagios\@comp:/srv/nagios/libexec/contrib# ./check_openweathermap.pl Wuerzburg,de sys-sunrise,sys-sunset,main-pressure,main-temp,main-humidity,wind-gust,wind-speed,wind-deg,weather-main,coord-lat,coord-lon,clouds-all,id,name
#OK - sys-sunrise=1374983154, sys-sunset=1375038455, main-pressure=1012, main-temp=298.69, main-humidity=38, wind-gust=2.06, wind-speed=1.03, wind-deg=0, weather-main=Clouds, coord-lat=49.787781, coord-lon=9.93611, clouds-all=68, id=2805615, name=Wuerzburg | sys-sunrise=1374983154;;;; sys-sunset=1375038455;;;; main-pressure=1012;;;; main-temp=298.69;;;; main-humidity=38;;;; wind-gust=2.06;;;; wind-speed=1.03;;;; wind-deg=0;;;; coord-lat=49.787781;;;; coord-lon=9.93611;;;; clouds-all=68;;;; id=2805615;;;
#nagios\@comp:/srv/nagios/libexec/contrib
#";
#  print "\n\n";
  ## nagios setup info
  print "## NAGIOS COMMANDS SETUP\n";
  print 'define command{
  command_name  check_openweathermap
  command_line  $USER1$/contrib/check_openweathermap.pl $ARG1$ $ARG2$ $ARG3$ $ARG4$ $ARG5$ $ARG6$ $ARG7$
}

';
  print "## NAGIOS SERVICE SETUP\n";
  print "define service{
  name                     owm-service
  use                      local-service
  normal_check_interval    10
  retry_check_interval     5
  register  0
}
            
define service{
  use                    owm-service         ; or owm-service-pnp, if you have pnp4nagios integrated and use that templatename
  host_name              yourhost
  service_description    Weather myplace
  check_command          check_openweathermap!London,uk!name,temp-current,humidity-current,wind-current,clouds,pressure-current
}


For the pnp4nagios related files (check_command entry as well as the graph-template), check the archive you got this script in.

";

  ## licensing info
  print "$PROG_NAME v$PROG_VERSION is licensed under Creative Commons Attribution-ShareAlike 3.0 Unported License.\n";
  print "There is no warranty of any kind, explicit or implied, for anything this software does or does not do.\n";
  print "\n";
  print "(c) 2013 by Frederic Krueger / check-openweathermap\@dev.techno.holics.at\n";
  print "\n";

  if ((defined($msg)) and ($msg ne ""))
  {
    print "\nERROR: $msg\n\n";
  }

  exit($ERRORS{"UNKNOWN"});
} # end sub usage


sub nagexit
{
  my ($nagstate, $msg, $perfdata, $debugout) = @_;
  $perfdata = ""  if (!defined($perfdata));
  $nagstate = "UNKNOWN" if ((!defined($nagstate)) or (!defined($ERRORS{uc($nagstate)})));
  $nagstate = uc($nagstate);
  $msg = $nagstate  if (!defined($msg));
  $debugout = ""  if (!defined($debugout));

  print "$msg" .($perfdata ne "" ? " | $perfdata" : "") ."\n";
  if ($debugout ne "") { print "$debugout\n"; }                 # add debug out, only used with errors

  exit ($ERRORS{$nagstate});
} # end sub nagexit



## args
if ((!defined($ARGV[0])) or (!defined($ARGV[1])))
{
  usage();
} # end if not enough arguments gotten

my $location   = $ARGV[0];
my $wantedinfo = $ARGV[1];

my $owmid = -1;
if ($location =~ /^id=(\d+)$/i)
{
  $owmid = $1;
}

my $opttype = "";
my $usedurl = "";
if ((defined($owmid)) and ($owmid >= 0))
{
  $opttype = "id";
  $usedurl = $idurl;
  $usedurl =~ s/<ID>/$owmid/isg;
} # end if use location as owmid
else
{
  $opttype = "loc";
  $usedurl = $locurl;
  $usedurl =~ s/<LOCATION>/$location/isg;
} # end if use location as city,country

my $weatherinfo_json = get("$usedurl");
if (!defined($weatherinfo_json))
{
  usage ("Problem getting weatherinfo for argument '$location' (url: $usedurl)\n$weatherinfo_json\n-------end-of-html-output-------");
}


# now JSON-parse the data gotten (hopefully)
my $wiref;
eval {
  $wiref = decode_json($weatherinfo_json);
};
if ( ($@) or ((!defined($wiref)) or (ref($wiref) ne "HASH")) )
{
  nagexit ("UNKNOWN", "UNKNOWN - Problem parsing returned JSON for location '$location'", "", "$@");
}


my $datastr = "";
my $perfdatastr = "";

if (($wantedinfo ne "") and (defined($opttranslation{$opttype})))
{
  my @wantedinfos = split /,/, $wantedinfo;
  my $val = ""; my $key = ""; my $hashpoint = ""; my $perfdataname = "";
  my $hpbase = $opttranslation{$opttype}{'_BASE_'};
  foreach $inp1key (@wantedinfos)
  {
#    print "- inp1key '$inp1key'\n";
    $perfdataname = $inp1key;
    $hashpoint = $hpbase;  $val = "";
    if (defined($opttranslation{$opttype}{$inp1key}))
    {
      $inp1key = $opttranslation{$opttype}{$inp1key};
    }

    my @keywordinfos = split /-/, $inp1key;
    foreach my $inp2key (@keywordinfos)
    {
#      print "-- inp2key '$inp2key'\n";
      if ($inp2key =~ /^\d+$/)
      {
        $inp2key = "[$inp2key]";    # get numbered list element
      }
      else
      {
        $inp2key = "{'$inp2key'}";
      }

      $hashpoint = "${hashpoint}$inp2key";
    } # end foreach keyword info part
#    print "hashpoint '$hashpoint'\n";

#    print "wantedinfo became $key\n";
    eval "\$val = \${\$wiref}$hashpoint;";
    if ((defined($val)) and ($val ne ""))
    {
      # if is a temperature, subtract zero degree kelvin (-273.15)
      if (($opttype eq "id") and ($hashpoint =~ /'temp/)) { $val -= 273; }
#      print "hashpoint $hashpoint => val '$val'\n";
      $datastr .= "$perfdataname=$val, ";
      if ($val =~ /^[0-9\.\-]+$/) { $perfdatastr .= "$perfdataname=$val;;;; "; }   # add perfdata only if numeric data gotten
    }
  } # end foreach key to translate

  if (length($datastr) > 0) { $datastr = substr($datastr, 0, length($datastr)-2); }
  if (length($perfdatastr) > 0) { $perfdatastr = substr($perfdatastr, 0, length($perfdatastr)-2); }
}

if ((length($datastr) > 0) and (length($perfdatastr) > 0))
{
  nagexit ("OK", "OK - $datastr", $perfdatastr, "");
}


# default exitcode:
my $wiref_ds = Dumper($wiref);
nagexit ("UNKNOWN", "UNKNOWN - Unknown error occured for location '$location' (url: $usedurl)", "", "Returned datastructure is:\n$wiref_ds");
